@@ -23,10 +23,10 @@
 #include "responsepacket.h"
 #include "requestpacket.h"
 #include "vnsicommand.h"
-#include "platform/util/StdString.h"
+#include "p8-platform/util/StdString.h"
 
 using namespace ADDON;
-using namespace PLATFORM;
+using namespace P8PLATFORM;
 
 cVNSIData::SMessage &
 cVNSIData::Queue::Enqueue(uint32_t serial)
@@ -61,14 +61,22 @@ cVNSIData::cVNSIData()
 
 cVNSIData::~cVNSIData()
 {
-  StopThread();
+  m_abort = true;
+  StopThread(0);
   Close();
 }
 
-bool cVNSIData::Open(const std::string& hostname, int port, const char* name, const std::string& mac)
+bool cVNSIData::Start(const std::string& hostname, int port, const char* name, const std::string& mac)
 {
-  /* First wake up the VDR server in case a MAC-Address is specified */
-  if (!mac.empty()) {
+  m_hostname = hostname;
+  m_port = port;
+
+  if (name != nullptr)
+    m_name = name;
+
+  // First wake up the VDR server in case a MAC-Address is specified
+  if (!mac.empty())
+  {
     const char* temp_mac;
     temp_mac = mac.c_str();
 
@@ -78,32 +86,25 @@ bool cVNSIData::Open(const std::string& hostname, int port, const char* name, co
     }
   }
 
-  if(!cVNSISession::Open(hostname, port, name))
-    return false;
-
-  return true;
-}
-
-bool cVNSIData::Login()
-{
-  if(!cVNSISession::Login())
-    return false;
+  PVR->ConnectionStateChange("VNSI started", PVR_CONNECTION_STATE_CONNECTING, "VNSI started");
 
+  m_abort = false;
+  m_connectionLost = true;
   CreateThread();
+
   return true;
 }
 
 void cVNSIData::OnDisconnect()
 {
-  XBMC->QueueNotification(QUEUE_ERROR, XBMC->GetLocalizedString(30044));
-  PVR->TriggerTimerUpdate();
+  PVR->ConnectionStateChange("vnsi connection lost", PVR_CONNECTION_STATE_DISCONNECTED, XBMC->GetLocalizedString(30044));
 }
 
 void cVNSIData::OnReconnect()
 {
-  XBMC->QueueNotification(QUEUE_INFO, XBMC->GetLocalizedString(30045));
+  EnableStatusInterface(true, false);
 
-  EnableStatusInterface(g_bHandleMessages);
+  PVR->ConnectionStateChange("vnsi connection established", PVR_CONNECTION_STATE_CONNECTED, XBMC->GetLocalizedString(30045));
 
   PVR->TriggerChannelUpdate();
   PVR->TriggerTimerUpdate();
@@ -187,12 +188,18 @@ bool cVNSIData::SupportRecordingsUndelete()
   return false;
 }
 
-bool cVNSIData::EnableStatusInterface(bool onOff)
+bool cVNSIData::EnableStatusInterface(bool onOff, bool wait)
 {
   cRequestPacket vrp;
   vrp.init(VNSI_ENABLESTATUSINTERFACE);
   vrp.add_U8(onOff);
 
+  if (!wait)
+  {
+    cVNSISession::TransmitMessage(&vrp);
+    return true;
+  }
+
   auto vresp = ReadResult(&vrp);
   if (!vresp)
   {
@@ -359,8 +366,10 @@ PVR_ERROR cVNSIData::GetTimerInfo(unsigned int timernumber, PVR_TIMER &tag)
       return PVR_ERROR_SERVER_ERROR;
   }
 
-  /* TODO: Implement own timer types to get support for the timer features introduced with PVR API 1.9.7 */
-  tag.iTimerType = PVR_TIMER_TYPE_NONE;
+  if (GetProtocol() >= 9)
+  {
+    tag.iTimerType = vresp->extract_U32();
+  }
 
   tag.iClientIndex      = vresp->extract_U32();
   int iActive           = vresp->extract_U32();
@@ -371,7 +380,7 @@ PVR_ERROR cVNSIData::GetTimerInfo(unsigned int timernumber, PVR_TIMER &tag)
   else if (iPending || iActive)
     tag.state = PVR_TIMER_STATE_SCHEDULED;
   else
-    tag.state = PVR_TIMER_STATE_CANCELLED;
+    tag.state = PVR_TIMER_STATE_DISABLED;
   tag.iPriority         = vresp->extract_U32();
   tag.iLifetime         = vresp->extract_U32();
                           vresp->extract_U32(); // channel number - unused
@@ -383,6 +392,12 @@ PVR_ERROR cVNSIData::GetTimerInfo(unsigned int timernumber, PVR_TIMER &tag)
   char *strTitle = vresp->extract_String();
   strncpy(tag.strTitle, strTitle, sizeof(tag.strTitle) - 1);
 
+  if (GetProtocol() >= 9)
+  {
+    char *epgSearch = vresp->extract_String();
+    strncpy(tag.strEpgSearchString, epgSearch, sizeof(tag.strEpgSearchString) - 1);
+  }
+
   return PVR_ERROR_NO_ERROR;
 }
 
@@ -406,8 +421,10 @@ bool cVNSIData::GetTimersList(ADDON_HANDLE handle)
       PVR_TIMER tag;
       memset(&tag, 0, sizeof(tag));
 
-      /* TODO: Implement own timer types to get support for the timer features introduced with PVR API 1.9.7 */
-      tag.iTimerType = PVR_TIMER_TYPE_NONE;
+      if (GetProtocol() >= 9)
+      {
+        tag.iTimerType = vresp->extract_U32();
+      }
 
       tag.iClientIndex      = vresp->extract_U32();
       int iActive           = vresp->extract_U32();
@@ -418,7 +435,7 @@ bool cVNSIData::GetTimersList(ADDON_HANDLE handle)
       else if (iPending || iActive)
         tag.state = PVR_TIMER_STATE_SCHEDULED;
       else
-        tag.state = PVR_TIMER_STATE_CANCELLED;
+        tag.state = PVR_TIMER_STATE_DISABLED;
       tag.iPriority         = vresp->extract_U32();
       tag.iLifetime         = vresp->extract_U32();
                               vresp->extract_U32(); // channel number - unused
@@ -432,6 +449,17 @@ bool cVNSIData::GetTimersList(ADDON_HANDLE handle)
       tag.iMarginStart      = 0;
       tag.iMarginEnd        = 0;
 
+      if (GetProtocol() >= 9)
+      {
+        char *epgSearch = vresp->extract_String();
+        strncpy(tag.strEpgSearchString, epgSearch, sizeof(tag.strEpgSearchString) - 1);
+      }
+
+      if (tag.startTime == 0)
+	tag.bStartAnyTime = true;
+      if (tag.endTime == 0)
+	tag.bEndAnyTime = true;
+
       PVR->TransferTimerEntry(handle, &tag);
     }
   }
@@ -499,6 +527,11 @@ PVR_ERROR cVNSIData::AddTimer(const PVR_TIMER &timerinfo)
   uint32_t starttime = timerinfo.startTime - timerinfo.iMarginStart*60;
   uint32_t endtime = timerinfo.endTime + timerinfo.iMarginEnd*60;
 
+  if (GetProtocol() >= 9)
+  {
+    vrp.add_U32(timerinfo.iTimerType);
+  }
+
   vrp.add_U32(timerinfo.state == PVR_TIMER_STATE_SCHEDULED);
   vrp.add_U32(timerinfo.iPriority);
   vrp.add_U32(timerinfo.iLifetime);
@@ -508,7 +541,12 @@ PVR_ERROR cVNSIData::AddTimer(const PVR_TIMER &timerinfo)
   vrp.add_U32(timerinfo.iWeekdays != PVR_WEEKDAY_NONE ? timerinfo.firstDay : 0);
   vrp.add_U32(timerinfo.iWeekdays);
   vrp.add_String(path.c_str());
-  vrp.add_String("");
+  vrp.add_String(timerinfo.strTitle);
+
+  if (GetProtocol() >= 9)
+  {
+    vrp.add_String(timerinfo.strEpgSearchString);
+  }
 
   auto vresp = ReadResult(&vrp);
   if (vresp == NULL || vresp->noResponse())
@@ -572,7 +610,12 @@ PVR_ERROR cVNSIData::UpdateTimer(const PVR_TIMER &timerinfo)
 
   cRequestPacket vrp;
   vrp.init(VNSI_TIMER_UPDATE);
+
   vrp.add_U32(timerinfo.iClientIndex);
+  if (GetProtocol() >= 9)
+  {
+    vrp.add_U32(timerinfo.iTimerType);
+  }
   vrp.add_U32(timerinfo.state == PVR_TIMER_STATE_SCHEDULED);
   vrp.add_U32(timerinfo.iPriority);
   vrp.add_U32(timerinfo.iLifetime);
@@ -581,8 +624,13 @@ PVR_ERROR cVNSIData::UpdateTimer(const PVR_TIMER &timerinfo)
   vrp.add_U32(endtime);
   vrp.add_U32(timerinfo.iWeekdays != PVR_WEEKDAY_NONE ? timerinfo.firstDay : 0);
   vrp.add_U32(timerinfo.iWeekdays);
-  vrp.add_String(timerinfo.strTitle);
   vrp.add_String("");
+  vrp.add_String(timerinfo.strTitle);
+
+  if (GetProtocol() >= 9)
+  {
+    vrp.add_String(timerinfo.strEpgSearchString);
+  }
 
   auto vresp = ReadResult(&vrp);
   if (vresp == NULL || vresp->noResponse())
@@ -600,6 +648,101 @@ PVR_ERROR cVNSIData::UpdateTimer(const PVR_TIMER &timerinfo)
   return PVR_ERROR_NO_ERROR;
 }
 
+PVR_ERROR cVNSIData::GetTimerTypes(PVR_TIMER_TYPE types[], int *size)
+{
+  *size = 0;
+  // One-shot manual
+  memset(&types[*size], 0, sizeof(types[*size]));
+  types[*size].iId = VNSI_TIMER_TYPE_MAN;
+  strncpy(types[*size].strDescription, XBMC->GetLocalizedString(30200), 64);
+  types[*size].iAttributes = PVR_TIMER_TYPE_IS_MANUAL               |
+                             PVR_TIMER_TYPE_SUPPORTS_ENABLE_DISABLE |
+                             PVR_TIMER_TYPE_SUPPORTS_CHANNELS       |
+                             PVR_TIMER_TYPE_SUPPORTS_START_TIME     |
+                             PVR_TIMER_TYPE_SUPPORTS_END_TIME       |
+                             PVR_TIMER_TYPE_SUPPORTS_PRIORITY       |
+                             PVR_TIMER_TYPE_SUPPORTS_LIFETIME       |
+                             PVR_TIMER_TYPE_SUPPORTS_RECORDING_FOLDERS;
+
+  (*size)++;
+
+  // Repeating manual
+  memset(&types[*size], 0, sizeof(types[*size]));
+  types[*size].iId = VNSI_TIMER_TYPE_MAN_REPEAT;
+  strncpy(types[*size].strDescription, XBMC->GetLocalizedString(30201), 64);
+  types[*size].iAttributes = PVR_TIMER_TYPE_IS_MANUAL               |
+                             PVR_TIMER_TYPE_IS_REPEATING            |
+                             PVR_TIMER_TYPE_SUPPORTS_ENABLE_DISABLE |
+                             PVR_TIMER_TYPE_SUPPORTS_CHANNELS       |
+                             PVR_TIMER_TYPE_SUPPORTS_START_TIME     |
+                             PVR_TIMER_TYPE_SUPPORTS_END_TIME       |
+                             PVR_TIMER_TYPE_SUPPORTS_PRIORITY       |
+                             PVR_TIMER_TYPE_SUPPORTS_LIFETIME       |
+                             PVR_TIMER_TYPE_SUPPORTS_FIRST_DAY      |
+                             PVR_TIMER_TYPE_SUPPORTS_WEEKDAYS       |
+                             PVR_TIMER_TYPE_SUPPORTS_RECORDING_FOLDERS;
+  (*size)++;
+
+  // One-shot epg-based
+  memset(&types[*size], 0, sizeof(types[*size]));
+  types[*size].iId = VNSI_TIMER_TYPE_EPG;
+  strncpy(types[*size].strDescription, XBMC->GetLocalizedString(30202), 64);
+  types[*size].iAttributes = PVR_TIMER_TYPE_SUPPORTS_ENABLE_DISABLE    |
+                             PVR_TIMER_TYPE_REQUIRES_EPG_TAG_ON_CREATE |
+                             PVR_TIMER_TYPE_SUPPORTS_CHANNELS          |
+                             PVR_TIMER_TYPE_SUPPORTS_START_TIME        |
+                             PVR_TIMER_TYPE_SUPPORTS_END_TIME          |
+                             PVR_TIMER_TYPE_SUPPORTS_PRIORITY          |
+                             PVR_TIMER_TYPE_SUPPORTS_LIFETIME          |
+                             PVR_TIMER_TYPE_SUPPORTS_RECORDING_FOLDERS;
+  (*size)++;
+
+  // addition timer supported by backend
+  if (GetProtocol() >= 9)
+  {
+    cRequestPacket vrp;
+    vrp.init(VNSI_TIMER_GETTYPES);
+    auto vresp = ReadResult(&vrp);
+    if (!vresp)
+    {
+      XBMC->Log(LOG_ERROR, "%s - Can't get response packed", __FUNCTION__);
+      return PVR_ERROR_NO_ERROR;
+    }
+    uint32_t vnsitimers = vresp->extract_U32();
+
+    if (vnsitimers & VNSI_TIMER_TYPE_EPG_SEARCH)
+    {
+      // EPG search timer
+      memset(&types[*size], 0, sizeof(types[*size]));
+      types[*size].iId = VNSI_TIMER_TYPE_EPG_SEARCH;
+      strncpy(types[*size].strDescription, XBMC->GetLocalizedString(30204), 64);
+      types[*size].iAttributes = PVR_TIMER_TYPE_IS_REPEATING |
+                                 PVR_TIMER_TYPE_SUPPORTS_ENABLE_DISABLE |
+                                 PVR_TIMER_TYPE_SUPPORTS_CHANNELS |
+                                 PVR_TIMER_TYPE_SUPPORTS_TITLE_EPG_MATCH |
+                                 PVR_TIMER_TYPE_SUPPORTS_PRIORITY |
+                                 PVR_TIMER_TYPE_SUPPORTS_LIFETIME;
+      (*size)++;
+    }
+
+    // VPS Timer
+    memset(&types[*size], 0, sizeof(types[*size]));
+    types[*size].iId = VNSI_TIMER_TYPE_VPS;
+    strncpy(types[*size].strDescription, XBMC->GetLocalizedString(30203), 64);
+    types[*size].iAttributes = PVR_TIMER_TYPE_IS_MANUAL |
+                               PVR_TIMER_TYPE_SUPPORTS_ENABLE_DISABLE |
+                               PVR_TIMER_TYPE_SUPPORTS_CHANNELS       |
+                               PVR_TIMER_TYPE_SUPPORTS_START_TIME     |
+                               PVR_TIMER_TYPE_SUPPORTS_END_TIME       |
+                               PVR_TIMER_TYPE_SUPPORTS_PRIORITY       |
+                               PVR_TIMER_TYPE_SUPPORTS_LIFETIME       |
+                               PVR_TIMER_TYPE_SUPPORTS_RECORDING_FOLDERS;
+    (*size)++;
+  }
+
+  return PVR_ERROR_NO_ERROR;
+}
+
 int cVNSIData::GetRecordingsCount()
 {
   cRequestPacket vrp;
@@ -641,6 +784,25 @@ PVR_ERROR cVNSIData::GetRecordingsList(ADDON_HANDLE handle)
 
     char *strChannelName = vresp->extract_String();
     strncpy(tag.strChannelName, strChannelName, sizeof(tag.strChannelName) - 1);
+    if (GetProtocol() >= 9)
+    {
+      tag.iChannelUid = -1;
+      uint32_t uuid = vresp->extract_U32();
+      if (uuid > 0)
+        tag.iChannelUid = uuid;
+      uint8_t type = vresp->extract_U8();
+      if (type == 1)
+	tag.iChannelUid = PVR_RECORDING_CHANNEL_TYPE_TV;
+      else if (type == 2)
+	tag.iChannelUid = PVR_RECORDING_CHANNEL_TYPE_RADIO;
+      else
+	tag.channelType = PVR_RECORDING_CHANNEL_TYPE_UNKNOWN;
+    }
+    else
+    {
+      tag.iChannelUid = PVR_CHANNEL_INVALID_UID;
+      tag.channelType = PVR_RECORDING_CHANNEL_TYPE_UNKNOWN;
+    }
 
     char *strTitle = vresp->extract_String();
     strncpy(tag.strTitle, strTitle, sizeof(tag.strTitle) - 1);
@@ -787,6 +949,12 @@ PVR_ERROR cVNSIData::GetDeletedRecordingsList(ADDON_HANDLE handle)
 
     char *strChannelName = vresp->extract_String();
     strncpy(tag.strChannelName, strChannelName, sizeof(tag.strChannelName) - 1);
+    if (GetProtocol() >= 9)
+    {
+      tag.iChannelUid = vresp->extract_S32();
+    }
+    else
+      tag.iChannelUid = PVR_CHANNEL_INVALID_UID;
 
     char *strTitle = vresp->extract_String();
     strncpy(tag.strTitle, strTitle, sizeof(tag.strTitle) - 1);
@@ -882,10 +1050,20 @@ void *cVNSIData::Process()
   while (!IsStopped())
   {
     // try to reconnect
-    if(ConnectionLost() && !TryReconnect())
+    if (m_connectionLost)
     {
-      Sleep(1000);
-      continue;
+      cVNSISession::eCONNECTIONSTATE state = TryReconnect();
+      if (state != cVNSISession::CONN_ESABLISHED)
+      {
+	if (state == cVNSISession::CONN_HOST_NOT_REACHABLE)
+	{
+	  PVR->ConnectionStateChange("vnsi server not reacheable",
+				     PVR_CONNECTION_STATE_SERVER_UNREACHABLE, nullptr);
+	}
+
+	Sleep(1000);
+	continue;
+      }
     }
 
     // if there's anything in the buffer, read it
