@@ -20,13 +20,14 @@
  */
 
 #include "client.h"
-#include "kodi/xbmc_pvr_dll.h"
+#include "xbmc_pvr_dll.h"
 #include "VNSIDemux.h"
 #include "VNSIRecording.h"
 #include "VNSIData.h"
 #include "VNSIChannelScan.h"
 #include "VNSIAdmin.h"
-#include "platform/util/util.h"
+#include "vnsicommand.h"
+#include "p8-platform/util/util.h"
 
 #include <sstream>
 #include <string>
@@ -34,9 +35,9 @@
 
 using namespace std;
 using namespace ADDON;
-using namespace PLATFORM;
+using namespace P8PLATFORM;
 
-ADDON_STATUS m_CurStatus      = ADDON_STATUS_UNKNOWN;
+ADDON_STATUS m_CurStatus = ADDON_STATUS_UNKNOWN;
 
 /* User adjustable settings are saved here.
  * Default values are defined inside client.h
@@ -46,27 +47,28 @@ std::string   g_szHostname              = DEFAULT_HOST;
 std::string   g_szWolMac                = "";
 int           g_iPort                   = DEFAULT_PORT;
 bool          g_bCharsetConv            = DEFAULT_CHARCONV;     ///< Convert VDR's incoming strings to UTF8 character set
-bool          g_bHandleMessages         = DEFAULT_HANDLE_MSG;   ///< Send VDR's OSD status messages to XBMC OSD
 int           g_iConnectTimeout         = DEFAULT_TIMEOUT;      ///< The Socket connection timeout
 int           g_iPriority               = DEFAULT_PRIORITY;     ///< The Priority this client have in response to other clients
 bool          g_bAutoChannelGroups      = DEFAULT_AUTOGROUPS;
 int           g_iTimeshift              = 1;
 std::string   g_szIconPath              = "";
 
-CHelper_libXBMC_addon *XBMC   = NULL;
-CHelper_libXBMC_codec *CODEC  = NULL;
-CHelper_libKODI_guilib *GUI    = NULL;
-CHelper_libXBMC_pvr   *PVR    = NULL;
+int prioVals[] = {0,5,10,15,20,25,30,35,40,45,50,55,60,65,70,75,80,85,90,95,99,100};
 
-cVNSIDemux      *VNSIDemuxer       = NULL;
-cVNSIData       *VNSIData          = NULL;
-cVNSIRecording  *VNSIRecording     = NULL;
+CHelper_libXBMC_addon *XBMC = nullptr;
+CHelper_libXBMC_codec *CODEC = nullptr;
+CHelper_libKODI_guilib *GUI = nullptr;
+CHelper_libXBMC_pvr *PVR = nullptr;
+
+cVNSIDemux *VNSIDemuxer = nullptr;
+cVNSIData *VNSIData = nullptr;
+cVNSIRecording *VNSIRecording = nullptr;
 
 bool IsTimeshift;
 time_t TimeshiftStartTime;
 time_t TimeshiftEndTime;
 time_t TimeshiftPlayTime;
-PLATFORM::CMutex TimeshiftMutex;
+P8PLATFORM::CMutex TimeshiftMutex;
 
 extern "C" {
 
@@ -161,7 +163,6 @@ ADDON_STATUS ADDON_Create(void* hdl, void* props)
     XBMC->Log(LOG_ERROR, "Couldn't get 'priority' setting, falling back to %i as default", -1);
     prio = DEFAULT_PRIORITY;
   }
-  int prioVals[] = {-99,-1,0,5,10,15,20,25,30,35,40,45,50,55,60,65,70,75,80,85,90,95,99,100};
   g_iPriority = prioVals[prio];
 
   /* Read setting "timeshift" from settings.xml */
@@ -188,14 +189,6 @@ ADDON_STATUS ADDON_Create(void* hdl, void* props)
     g_iConnectTimeout = DEFAULT_TIMEOUT;
   }
 
-  /* Read setting "handlemessages" from settings.xml */
-  if (!XBMC->GetSetting("handlemessages", &g_bHandleMessages))
-  {
-    /* If setting is unknown fallback to defaults */
-    XBMC->Log(LOG_ERROR, "Couldn't get 'handlemessages' setting, falling back to 'true' as default");
-    g_bHandleMessages = DEFAULT_HANDLE_MSG;
-  }
-
   /* Read setting "autochannelgroups" from settings.xml */
   if (!XBMC->GetSetting("autochannelgroups", &g_bAutoChannelGroups))
   {
@@ -218,29 +211,19 @@ ADDON_STATUS ADDON_Create(void* hdl, void* props)
   }
   free(buffer);
 
-  try {
+  try
+  {
     VNSIData = new cVNSIData;
-    if (!VNSIData->Open(g_szHostname, g_iPort, NULL, g_szWolMac))
+    m_CurStatus = ADDON_STATUS_OK;
+    if (!VNSIData->Start(g_szHostname, g_iPort, nullptr, g_szWolMac))
     {
       ADDON_Destroy();
-      m_CurStatus = ADDON_STATUS_LOST_CONNECTION;
+      m_CurStatus = ADDON_STATUS_PERMANENT_FAILURE;
       return m_CurStatus;
     }
-
-    if (!VNSIData->Login())
-    {
-      ADDON_Destroy();
-      m_CurStatus = ADDON_STATUS_LOST_CONNECTION;
-      return m_CurStatus;
-    }
-
-    if (!VNSIData->EnableStatusInterface(g_bHandleMessages))
-    {
-      ADDON_Destroy();
-      m_CurStatus = ADDON_STATUS_LOST_CONNECTION;
-      return m_CurStatus;
-    }
-  } catch (std::exception e) {
+  }
+  catch (std::exception e)
+  {
     XBMC->Log(LOG_ERROR, "%s - %s", __FUNCTION__, e.what());
     ADDON_Destroy();
     m_CurStatus = ADDON_STATUS_LOST_CONNECTION;
@@ -253,7 +236,6 @@ ADDON_STATUS ADDON_Create(void* hdl, void* props)
   hook.iLocalizedStringId = 30107;
   PVR->AddMenuHook(&hook);
 
-  m_CurStatus = ADDON_STATUS_OK;
   return m_CurStatus;
 }
 
@@ -321,17 +303,18 @@ ADDON_STATUS ADDON_SetSetting(const char *settingName, const void *settingValue)
   }
   else if (str == "port")
   {
-    XBMC->Log(LOG_INFO, "Changed Setting 'port' from %u to %u", g_iPort, *(int*) settingValue);
-    if (g_iPort != *(int*) settingValue)
+    XBMC->Log(LOG_INFO, "Changed Setting 'port' from %u to %u", g_iPort, *(int*)settingValue);
+    if (g_iPort != *(int*)settingValue)
     {
-      g_iPort = *(int*) settingValue;
+      g_iPort = *(int*)settingValue;
       return ADDON_STATUS_NEED_RESTART;
     }
   }
   else if (str == "priority")
   {
-    XBMC->Log(LOG_INFO, "Changed Setting 'priority' from %u to %u", g_iPriority, *(int*) settingValue);
-    g_iPriority = *(int*) settingValue;
+    int newPrio = prioVals[*(int*)settingValue];
+    XBMC->Log(LOG_INFO, "Changed Setting 'priority' from %u to %u", g_iPriority, newPrio);
+    g_iPriority = newPrio;
   }
   else if (str == "timeshift")
   {
@@ -348,12 +331,6 @@ ADDON_STATUS ADDON_SetSetting(const char *settingName, const void *settingValue)
     XBMC->Log(LOG_INFO, "Changed Setting 'timeout' from %u to %u", g_iConnectTimeout, *(int*) settingValue);
     g_iConnectTimeout = *(int*) settingValue;
   }
-  else if (str == "handlemessages")
-  {
-    XBMC->Log(LOG_INFO, "Changed Setting 'handlemessages' from %u to %u", g_bHandleMessages, *(bool*) settingValue);
-    g_bHandleMessages = *(bool*) settingValue;
-    if (VNSIData) VNSIData->EnableStatusInterface(g_bHandleMessages);
-  }
   else if (str == "autochannelgroups")
   {
     XBMC->Log(LOG_INFO, "Changed Setting 'autochannelgroups' from %u to %u", g_bAutoChannelGroups, *(bool*) settingValue);
@@ -376,14 +353,30 @@ void ADDON_FreeSettings()
 
 }
 
-void ADDON_Announce(const char *flag, const char *sender, const char *message, const void *data)
-{
-}
-
 /***********************************************************
  * PVR Client AddOn specific public library functions
  ***********************************************************/
 
+void OnSystemSleep()
+{
+}
+
+void OnSystemWake()
+{
+  if (XBMC && !g_szWolMac.empty())
+  {
+    XBMC->WakeOnLan(g_szWolMac.c_str());
+  }
+}
+
+void OnPowerSavingActivated()
+{
+}
+
+void OnPowerSavingDeactivated()
+{
+}
+
 const char* GetPVRAPIVersion(void)
 {
   static const char *strApiVersion = XBMC_PVR_API_VERSION;
@@ -530,7 +523,6 @@ PVR_ERROR GetChannels(ADDON_HANDLE handle, bool bRadio)
   }
 }
 
-
 /*******************************************/
 /** PVR Channelgroups Functions           **/
 
@@ -576,26 +568,39 @@ PVR_ERROR GetChannelGroupMembers(ADDON_HANDLE handle, const PVR_CHANNEL_GROUP &g
   }
 }
 
-
 /*******************************************/
 /** PVR Timer Functions                   **/
 
+
 PVR_ERROR GetTimerTypes(PVR_TIMER_TYPE types[], int *size)
 {
-  /* TODO: Implement this to get support for the timer features introduced with PVR API 1.9.7 */
-  return PVR_ERROR_NOT_IMPLEMENTED;
+  if (!VNSIData)
+    return PVR_ERROR_SERVER_ERROR;
+
+  try
+  {
+    return VNSIData->GetTimerTypes(types, size);
+  }
+  catch (std::exception e)
+  {
+    XBMC->Log(LOG_ERROR, "%s - %s", __FUNCTION__, e.what());
+    return PVR_ERROR_SERVER_ERROR;
+  }
 }
 
 int GetTimersAmount(void)
 {
   if (!VNSIData)
-    return 0;
+    return PVR_ERROR_SERVER_ERROR;
 
-  try {
+  try
+  {
     return VNSIData->GetTimersCount();
-  } catch (std::exception e) {
+  }
+  catch (std::exception e)
+  {
     XBMC->Log(LOG_ERROR, "%s - %s", __FUNCTION__, e.what());
-    return 0;
+    return PVR_ERROR_SERVER_ERROR;
   }
 }
 
@@ -604,10 +609,12 @@ PVR_ERROR GetTimers(ADDON_HANDLE handle)
   if (!VNSIData)
     return PVR_ERROR_SERVER_ERROR;
 
-  try {
-    /* TODO: Change implementation to get support for the timer features introduced with PVR API 1.9.7 */
+  try
+  {
     return (VNSIData->GetTimersList(handle) ? PVR_ERROR_NO_ERROR : PVR_ERROR_SERVER_ERROR);
-  } catch (std::exception e) {
+  }
+  catch (std::exception e)
+  {
     XBMC->Log(LOG_ERROR, "%s - %s", __FUNCTION__, e.what());
     return PVR_ERROR_SERVER_ERROR;
   }
@@ -618,9 +625,12 @@ PVR_ERROR AddTimer(const PVR_TIMER &timer)
   if (!VNSIData)
     return PVR_ERROR_SERVER_ERROR;
 
-  try {
+  try
+  {
     return VNSIData->AddTimer(timer);
-  } catch (std::exception e) {
+  }
+  catch (std::exception e)
+  {
     XBMC->Log(LOG_ERROR, "%s - %s", __FUNCTION__, e.what());
     return PVR_ERROR_SERVER_ERROR;
   }
@@ -631,9 +641,12 @@ PVR_ERROR DeleteTimer(const PVR_TIMER &timer, bool bForce)
   if (!VNSIData)
     return PVR_ERROR_SERVER_ERROR;
 
-  try {
+  try
+  {
     return VNSIData->DeleteTimer(timer, bForce);
-  } catch (std::exception e) {
+  }
+  catch (std::exception e)
+  {
     XBMC->Log(LOG_ERROR, "%s - %s", __FUNCTION__, e.what());
     return PVR_ERROR_SERVER_ERROR;
   }
@@ -644,9 +657,12 @@ PVR_ERROR UpdateTimer(const PVR_TIMER &timer)
   if (!VNSIData)
     return PVR_ERROR_SERVER_ERROR;
 
-  try {
+  try
+  {
     return VNSIData->UpdateTimer(timer);
-  } catch (std::exception e) {
+  }
+  catch (std::exception e)
+  {
     XBMC->Log(LOG_ERROR, "%s - %s", __FUNCTION__, e.what());
     return PVR_ERROR_SERVER_ERROR;
   }
@@ -747,10 +763,16 @@ bool OpenLiveStream(const PVR_CHANNEL &channel)
 {
   CloseLiveStream();
 
-  try {
+  try
+  {
     VNSIDemuxer = new cVNSIDemux;
+    TimeshiftStartTime = 0;
+    TimeshiftEndTime = 0;
+    TimeshiftPlayTime = 0;
     return VNSIDemuxer->OpenChannel(channel);
-  } catch (std::exception e) {
+  }
+  catch (std::exception e)
+  {
     XBMC->Log(LOG_ERROR, "%s - %s", __FUNCTION__, e.what());
     delete VNSIDemuxer;
     VNSIDemuxer = NULL;
@@ -779,7 +801,8 @@ PVR_ERROR GetStreamProperties(PVR_STREAM_PROPERTIES* pProperties)
 
 void DemuxAbort(void)
 {
-  if (VNSIDemuxer) VNSIDemuxer->Abort();
+  if (VNSIDemuxer)
+    VNSIDemuxer->Abort();
 }
 
 DemuxPacket* DemuxRead(void)
@@ -803,14 +826,6 @@ DemuxPacket* DemuxRead(void)
   return pkt;
 }
 
-int GetCurrentClientChannel(void)
-{
-  if (VNSIDemuxer)
-    return VNSIDemuxer->CurrentChannel();
-
-  return -1;
-}
-
 bool SwitchChannel(const PVR_CHANNEL &channel)
 {
   try {
@@ -852,10 +867,24 @@ bool CanSeekStream(void)
   return ret;
 }
 
+bool IsRealTimeStream()
+{
+  if (VNSIDemuxer)
+  {
+    const CLockObject lock(TimeshiftMutex);
+    if (!IsTimeshift)
+      return true;
+    if (TimeshiftEndTime - TimeshiftPlayTime < 10)
+      return true;
+  }
+  return false;
+}
+
 bool SeekTime(int time, bool backwards, double *startpts)
 {
   bool ret = false;
-  try {
+  try
+  {
     if (VNSIDemuxer)
       ret = VNSIDemuxer->SeekTime(time, backwards, startpts);
   } catch (std::exception e) {
@@ -870,7 +899,7 @@ time_t GetPlayingTime()
   if (VNSIDemuxer)
   {
     const CLockObject lock(TimeshiftMutex);
-	time = TimeshiftPlayTime;
+    time = TimeshiftPlayTime;
   }
   return time;
 }
@@ -881,7 +910,7 @@ time_t GetBufferTimeStart()
   if (VNSIDemuxer)
   {
     const CLockObject lock(TimeshiftMutex);
-	time = TimeshiftStartTime;
+    time = TimeshiftStartTime;
   }
   return time;
 }
@@ -892,7 +921,7 @@ time_t GetBufferTimeEnd()
   if (VNSIDemuxer)
   {
     const CLockObject lock(TimeshiftMutex);
-	time = TimeshiftEndTime;
+    time = TimeshiftEndTime;
   }
   return time;
 }
@@ -922,14 +951,17 @@ bool OpenRecordedStream(const PVR_RECORDING &recording)
   CloseRecordedStream();
 
   VNSIRecording = new cVNSIRecording;
-  try {
+  try
+  {
     return VNSIRecording->OpenRecording(recording);
-  } catch (std::exception e) {
+  }
+  catch (std::exception e)
+  {
     XBMC->Log(LOG_ERROR, "%s - %s", __FUNCTION__, e.what());
     VNSIRecording->Close();
     delete VNSIRecording;
     VNSIRecording = NULL;
-    return NULL;
+    return false;
   }
 }
 
@@ -1033,4 +1065,5 @@ PVR_ERROR SetRecordingPlayCount(const PVR_RECORDING &recording, int count) { ret
 PVR_ERROR SetRecordingLastPlayedPosition(const PVR_RECORDING &recording, int lastplayedposition) { return PVR_ERROR_NOT_IMPLEMENTED; }
 int GetRecordingLastPlayedPosition(const PVR_RECORDING &recording) { return -1; }
 unsigned int GetChannelSwitchDelay(void) { return 0; }
+PVR_ERROR SetEPGTimeFrame(int) { return PVR_ERROR_NOT_IMPLEMENTED; }
 }
